package Text::PDF;

$VERSION = '0.31';

1;

=head1 NAME

Text::PDF - Module for manipulating PDF files

=head1 DESCRIPTION

This module allows interaction with existing PDF files directly. It includes
various tools including:

  pdfbklt - make booklets out of existing PDF files
  pdfrevert - remove edits from a PDF file
  pdfstamp - stamp text on each page of a PDF file

various example programs are also included

=head1 AUTHOR

Martin Hosken Martin_Hosken@sil.org

Copyright Martin Hosken 1999 and onwards

No warranty or expression of effectiveness, least of all regarding anyone's
safety, is implied in this software or documentation.

=head2 COPYRIGHT

This Perl Text::PDF module is licensed under the same terms as Perl itself.


